from pathlib import Path

from fastapi import FastAPI
from starlette.staticfiles import StaticFiles

from s3serve.routes import buckets

app = FastAPI(title='S3 Serve Web')
app_api = FastAPI(title='S3 Serve API')

app.mount('/api/v1', app_api, name='api')
app.mount('/', StaticFiles(directory=Path(__file__).resolve().parent.joinpath('web', 'dist'), html=True), name='static')

app_api.include_router(buckets.router)



from enum import Enum

from pydantic_settings import BaseSettings, SettingsConfigDict


class Environment(Enum):
    LOCAL = 'local'
    PRODUCTION = "production"


class Settings(BaseSettings):
    model_config = SettingsConfigDict(env_file='.env', case_sensitive=False, env_file_encoding='utf-8')
    environment: Environment = Environment.PRODUCTION


config = Settings()



import io

import boto3
from fastapi import APIRouter
from starlette.responses import Response

from s3serve.utils import get_media_type

router = APIRouter()


@router.get("/buckets")
def get_buckets():
    s3 = boto3.client('s3')
    res = s3.list_buckets(MaxBuckets=1000)
    buckets = res['Buckets']
    return {"buckets": [bucket['Name'] for bucket in buckets]}


@router.get("/buckets/{bucket}/folders")
def get_folders(bucket: str, prefix: str = None):
    s3 = boto3.client('s3')
    if prefix:
        objects = s3.list_objects_v2(Bucket=bucket, Prefix=prefix, Delimiter='/')
    else:
        objects = s3.list_objects_v2(Bucket=bucket, Delimiter='/')
    common_prefixes = objects.get('CommonPrefixes', [])
    folders = []
    for common_prefix in common_prefixes:
        folder = [part for part in common_prefix['Prefix'].split('/') if part][-1]
        folders.append(folder)
    return {"folders": folders}


@router.get("/buckets/{bucket}/objects")
def get_objects(bucket: str, prefix: str = None):
    s3 = boto3.client('s3')
    if prefix:
        res = s3.list_objects_v2(Bucket=bucket, Prefix=prefix, MaxKeys=25, Delimiter='/')
    else:
        res = s3.list_objects_v2(Bucket=bucket, MaxKeys=25, Delimiter='/')
    objects = res['Contents'] if 'Contents' in res else []
    return {"objects": [obj['Key'] for obj in objects]}


@router.get("/buckets/{bucket}/object")
def get_object(bucket: str, key: str):
    s3 = boto3.client('s3')
    res = s3.get_object(Bucket=bucket, Key=key)
    body = res['Body']
    image_bytes = body.read()
    media_type = get_media_type(key)
    return Response(content=image_bytes, media_type=media_type)



import uvicorn

from s3serve.config import config, Environment


def main():
    reload = config.environment == Environment.LOCAL
    uvicorn.run('s3serve.app:app', host='0.0.0.0', port=8080, reload=reload, reload_dirs=['s3serve'])



def get_media_type(key: str) -> str:
    if key.endswith('.jpg'):
        return 'image/jpeg'
    elif key.endswith('.png'):
        return 'image/png'
    elif key.endswith('.txt'):
        return 'text/plain'
    else:
        return 'application/octet-stream'
